// File: Clipbord.Cc

#define GMEM_MOVEABLE   HTOI("2")
#define GMEM_DDESHARE   HTOI("2000")
#define CF_TEXT         1

CLASS TextClipboard
/*======================================================================

Version: 1.0
Programmer: Bowen Moursund (bowen@nerds.com)
Date: 11/22/97
Written for: Visual dBASE 7.0

------------------------------------------------------------------------

Purpose:
The programmatic transfer of text to and from the Windows clipboard and
VdB7. The class works only with text.

------------------------------------------------------------------------

Notes:
The class is an adaptation of the work of Jim Sare and Gary King. It
also relies upon a DLL created by Vic McClung. Many thanks, fellas!

------------------------------------------------------------------------

Revision History:
11/04/84: Gary King's Clip.Prg
11/22/97: TextClipboard ver. 1.0

------------------------------------------------------------------------

Example of use:

   set procedure to Clipbord.CC additive
   _app.Clip = new TextClipboard()
   _app.Clip.SetClipboardText("Did it work?")

======================================================================*/

	// You'll probably need to modify the path to StringEx.Cc
	//set procedure to ;
   //"c:\Borland\Visual dBASE\custom\StringEx.Cc" additive

	protect Text
   this.Text = ""

   // Prototype memory block copy functions from Vic McClung's DLL
  	if type( "CopyMemPut" ) <> "FP"
     	extern CVOID CopyMemPut( CLONG, CPTR, CLONG ) ;
        dbmem.dll from "CopyMem"
  	endif
   if type( "CopyMemGet" ) <> "FP"
      extern CVOID CopyMemGet( CPTR, CLONG, CLONG ) ;
         dbmem.dll from "CopyMem"
   endif
   // Many thanks to Vic!!

   // Prototype other required Win32API functions
   if type( "GlobalSize" ) <> "FP"
      extern CLONG GlobalSize( CHANDLE ) kernel32
   endif
   if type( "GlobalLock" ) <> "FP"
      extern CLONG GlobalLock( CHANDLE ) kernel32
   endif
   if type( "GlobalUnlock" ) <> "FP"
      extern CINT GlobalUnlock( CHANDLE ) kernel32
   endif
   if type( "GlobalAlloc" ) <> "FP"
      extern CINT GlobalAlloc(CINT, CLONG) kernel32
   endif
   if type( "CloseClipboard" ) <> "FP"
      extern CLOGICAL CloseClipboard ( CVOID ) user32
   endif
   if type( "EmptyClibBoard" ) <> "FP"
      extern CLOGICAL EmptyClipboard ( CVOID ) user32
   endif
   if type( "GetClipboardData" ) <> "FP"
      extern CHANDLE GetClipboardData ( CINT ) user32
   endif
   if type( "IsClipboardFormatAvailable" ) <> "FP"
      extern CLOGICAL IsClipboardFormatAvailable ( CINT ) user32
   endif
   if type( "OpenClipboard" ) <> "FP"
      extern CLOGICAL OpenClipboard ( CHANDLE ) user32
   endif
   if type( "SetClipboardData" ) <> "FP"
      extern CHANDLE SetClipboardData ( CINT,CHANDLE ) user32
   endif


   //  Use this function to set this.Text programmatically
   FUNCTION SetText
   	// Receive parm as private for type() check
   	parameter cStr

      local lReturn
      lReturn = false

      // Nothing but strings, thank you
      if type( "cStr" ) == "C"
      	this.Text = cStr
         lReturn = true
      endif

		RETURN lReturn

   //  Use this function to retrieve the contents of this.Text
	FUNCTION GetText
   	RETURN this.Text      		

   // Store the contents of the Windows clipboard to this.Text
   FUNCTION GetClipboardText
      local nSourceSize, nSourceAddress, hSource,;
            lClipBoardOpen, lHaveData

      lHaveData = false

      *-- Control loop
      do
         lClipBoardOpen = OpenClipboard( _app.Framewin.hWND )
         if not lClipBoardOpen
            exit
         endif

         // Get handle to block of text
         hSource = GetClipboardData( CF_TEXT )
         if hSource = 0
            exit
         endif

         // Get size of text block in bytes
         nSourceSize = GlobalSize( hSource )
         if nSourceSize = 0
            exit
         endif

         // Get address of text block
         nSourceAddress = GlobalLock( hSource )
         if nSourceAddress = 0
            exit
         endif

         local cBuffer, oStr
         store space( nSourceSize ) to cBuffer

         // Copy text block from clipboard to VdB buffer
         CopyMemGet( cBuffer, nSourceAddress, nSourceSize )

         // Unlock source when copy completed
         GlobalUnlock( hSource )

			// Instantiate custom VdB string object
         oStr = new StringExt()

         // Copy text buffer to string object
         oStr.string = cBuffer

         // Convert to VdB double-byte string
         oStr.SingleToDouble()

         // Truncate string at first null
         oStr.truncate()

         this.Text = oStr.string

         lHaveData = true
      until( true )

      if lClipBoardOpen
         CloseClipboard()
      endif

      RETURN lHaveData


   // Store the contents of this.Text to the Windows clipboard
   // Optionally pass a string directly to this method
   FUNCTION SetClipboardText( cStr )

   	//  Did we receive a parameter?
   	if argCount() > 0 and not class::SetText( cStr )
      	RETURN false
      endif

      local lSuccess, hDestination, nDestinationAddress,;
            oStr, nBufferLen

      lSuccess = false

      do
         // Any text to copy?
			if len( this.Text ) < 1
            exit
         endif

			// Instantiate custom VdB string object
         oStr = new StringExt()

         // Copy the contents of this,Text to it
         oStr.string = this.Text + chr(0)
         nBufferLen = oStr.length

         // Allocate a Windows memory buffer for the data and get handle 
         hDestination = GlobalAlloc( GMEM_MOVEABLE + GMEM_DDESHARE, nBufferLen)
         if hDestination = 0
            exit
         endif

         // Get address of memory buffer
         nDestinationAddress = GlobalLock(hDestination)
         if nDestinationAddress = 0
            exit
         endif

         // Convert VdB double-byte string to single-byte
         oStr.DoubleToSingle()

			// Copy single-byte string to Windows memory buffer
         CopyMemPut( nDestinationAddress, oStr.string, nBufferLen ) && Move data to mem buffer

         // Unlock destination when copy completed
         GlobalUnlock( hDestination )

         // Open the Windows clipboard
         if not OpenClipboard( _app.Framewin.hWND )
            exit
         endif

         EmptyClipboard()

         // Place the text buffer handle in clipboard
         lSuccess = ( SetClipboardData( CF_TEXT, hDestination ) <> 0 )

         CloseClipboard()
      until( true )

      RETURN lSuccess

ENDCLASS

CLASS StringExt of string
/*======================================================================

Description:

Version: 1.0
Programmer: Bowen Moursund (bowen@nerds.com)
Date: 11/22/97
Written for: Visual dBASE 7.0

CLASS StringExt extends the stock string class with a few
useful methods.

======================================================================*/

	// Truncate this.string at first null
	FUNCTION Truncate
	   local nPos
   	nPos = at(chr(0), this.string)
	   if nPos > 0
   		this.string = this.left(nPos-1)
	   endif
   	return this.string

   // Covert a single-byte this.string to a VdB double-byte string
   FUNCTION SingleToDouble
	   local oStr, n
   	oStr = new string()
	   oStr.string = space(this.length*2)
   	for n = 0 to this.length
			oStr.setByte(n*2, this.getByte(n))
	   next n
   	this.string = oStr.string
   	return this.string

   // Covert a VdB double-byte this.string to a single-byte string
   FUNCTION DoubleToSingle
	   local oStr, n
   	oStr = new string()
	   oStr.string = space(ceiling(this.length/2))
   	for n = 0 to (this.length*2-2) step 2
			oStr.setByte(n/2, this.getByte(n))
	   next n
   	this.string = oStr.string
   	return this.string

ENDCLASS

// EoF: Clipbord.Cc
